#!/bin/bash
#

# buildXmlrpc-c.sh <product_dir> <e2|e4> <debug|opt|prof> [tar] [source]

# IMPORTANT: If curl-devel (SLF5) or libcurl-devel (SLF6) is installed, 
#            the curl xml client will also be built

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e2|e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}/build
    tardir=${product_dir}/${package}/${pkgver}/tar
}

select_gcc()
{
    if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${basequal}" = "e4" ] ||  [ "${basequal}" = "gcc48" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    elif [ "${basequal}" = "e1" ] || [ "${basequal}" = "gcc46" ]
    then
       gccdotver=4.6
       gccver=v4_6_1
    else
      echo "ERROR: unrecognized qualifier ${basequal}"
      usage
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ ${gccshortver} = ${gccdotver} ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=xmlrpc_c
pkgver=v1_25_24
ssibuildshims_version=v0_06_03
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=xmlrpc-c-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [ "${basequal}" = "e2" ] || [ "${basequal}" = "e4" ]
then
  cxxflg="${cflg} -std=c++11 -Wno-deprecated-declarations"
else
  cxxflg="${cflg}"
fi

setup_gcc

have_curl=`rpm -qa | grep curl-devel | wc -l`
if [ ${have_curl} -gt 0 ]
then
   build_curl=--enable-curl-client 
   echo "building xmlrpc_c with curl client"
else
   build_curl=--disable-curl-client 
   echo "building xmlrpc_c WITHOUT curl client"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${blddir}
tar xf ${tardir}/${pkgtarname}.tgz

cd ${blddir}/${pkgtarname}

./configure --prefix=${pkgdir} \
            ${build_curl} --enable-cplusplus --enable-tools \
	    --disable-libwww-client --disable-wininet-client
make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}"
make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" check
make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" install 

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "xmlrpc_c libraries are in ${XMLRPC_C_LIBRARY}"

# fixing embedded path
mv ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak
cat ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak \
    | sed 's%^PREFIX=.*$%PREFIX=${XMLRPC_C_FQ_DIR}%' \
    | sed 's%^HEADERINST_DIR=.*$%HEADERINST_DIR=${XMLRPC_C_INCLUDE_DIR}%' \
    | sed 's%^LIBINST_DIR=.*$%LIBINST_DIR=${XMLRPC_C_LIBRARY}%' \
    | sed 's%^BLDDIR=.*$%BLDDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    | sed 's%^ABS_SRCDIR=.*$%ABS_SRCDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    > ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config
chmod +x ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
